/*
** Module   : Image.java
** Abstract : Definition of a IMAGE widget.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140805 Created initial version.  Not yet fully tested/complete.
** 002 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 003 EVL 20141208 Adding wiget initialization method.
** 004 EVL 20150123 Removed image source file name cached member.  This is just a double for what
**                  we have in image widget config.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * Definition of a IMAGE widget.
 */
public abstract class Image<O extends OutputManager<?>>
extends AbstractWidget<O>
{
   /** The image configuration. */
   private ImageConfig config;
   
   /**
    * Basic c'tor.
    * 
    * @param   id
    *          The widget's ID.
    */
   public Image(WidgetId id)
   {
      config = ConfigManager.getInstance().resolveWidgetConfig(id, ImageConfig.class);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * <p>
    * Call {@link #initialize} to finish the widget initialization.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public Image(WidgetId id, ImageConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, WidgetConfig cfg)
   {
      config = (ImageConfig) ConfigManager.getInstance().resolveWidgetConfig(id, cfg);
      ConfigManager.getInstance().replaceConfig(config, cfg);
   }

   /**
    * Get the widget config.
    * 
    * @return   See above.
    */
   @SuppressWarnings("unchecked")
   @Override
   public BaseConfig config()
   {
      return config;
   }
   
   /**
    * Get widget dimension.
    * 
    * @return  widget dimension.
    */
   @Override
   public Dimension dimension()
   {
      return new Dimension(this.width(), this.height());
   }

   /**
    * Get widget minimum size - same as {@link #dimension()}.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }
}
